"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const Promise = require("bluebird");
const ResinSdk = require("resin-sdk");
const deviceConfig = require("resin-device-config");
const semver = require("resin-semver");
const resin = ResinSdk.fromSharedOptions();
function readRootCa() {
    const caFile = process.env.NODE_EXTRA_CA_CERTS;
    if (!caFile) {
        return Promise.resolve();
    }
    return Promise.fromCallback(cb => fs.readFile(caFile, { encoding: 'utf8' }, cb))
        .then(pem => Buffer.from(pem).toString('base64'))
        .catch({ code: 'ENOENT' }, () => { });
}
function generateBaseConfig(application, options) {
    if (options.appUpdatePollInterval) {
        options = Object.assign({}, options, { appUpdatePollInterval: options.appUpdatePollInterval * 60 * 1000 });
    }
    return Promise.props({
        userId: resin.auth.getUserId(),
        username: resin.auth.whoami(),
        apiUrl: resin.settings.get('apiUrl'),
        vpnUrl: resin.settings.get('vpnUrl'),
        registryUrl: resin.settings.get('registryUrl'),
        deltaUrl: resin.settings.get('deltaUrl'),
        apiConfig: resin.models.config.getAll(),
        rootCA: readRootCa().catch(() => {
            console.warn('Could not read root CA');
        }),
    }).then(results => {
        return deviceConfig.generate({
            application,
            user: {
                id: results.userId,
                username: results.username,
            },
            endpoints: {
                api: results.apiUrl,
                vpn: results.vpnUrl,
                registry: results.registryUrl,
                delta: results.deltaUrl,
            },
            pubnub: results.apiConfig.pubnub,
            mixpanel: {
                token: results.apiConfig.mixpanelToken,
            },
            balenaRootCA: results.rootCA,
        }, options);
    });
}
exports.generateBaseConfig = generateBaseConfig;
function generateApplicationConfig(application, options) {
    return generateBaseConfig(application, options).tap(config => {
        if (semver.satisfies(options.version, '>=2.7.8')) {
            return addProvisioningKey(config, application.id);
        }
        else {
            return addApplicationKey(config, application.id);
        }
    });
}
exports.generateApplicationConfig = generateApplicationConfig;
function generateDeviceConfig(device, deviceApiKey, options) {
    return resin.models.application
        .get(device.belongs_to__application.__id)
        .then(application => {
        return generateBaseConfig(application, options).tap(config => {
            if (deviceApiKey) {
                return addDeviceKey(config, device.uuid, deviceApiKey);
            }
            else if (semver.satisfies(options.version, '>=2.0.3')) {
                return addDeviceKey(config, device.uuid, true);
            }
            else {
                return addApplicationKey(config, application.id);
            }
        });
    })
        .then(config => {
        config.registered_at = Math.floor(Date.now() / 1000);
        config.deviceId = device.id;
        config.uuid = device.uuid;
        return config;
    });
}
exports.generateDeviceConfig = generateDeviceConfig;
function addApplicationKey(config, applicationNameOrId) {
    return resin.models.application
        .generateApiKey(applicationNameOrId)
        .tap(apiKey => {
        config.apiKey = apiKey;
    });
}
function addProvisioningKey(config, applicationNameOrId) {
    return resin.models.application
        .generateProvisioningKey(applicationNameOrId)
        .tap(apiKey => {
        config.apiKey = apiKey;
    });
}
function addDeviceKey(config, uuid, customDeviceApiKey) {
    return Promise.try(() => {
        if (customDeviceApiKey === true) {
            return resin.models.device.generateDeviceKey(uuid);
        }
        else {
            return customDeviceApiKey;
        }
    }).tap(deviceApiKey => {
        config.deviceApiKey = deviceApiKey;
    });
}
//# sourceMappingURL=config.js.map